@@ -1042,14 +1042,13 @@ static uint_t Cookies_internal_dots_required(const char *host)
 
       if (tld_len > 0) {
          /* These TLDs were chosen by examining the current publicsuffix list
-          * in February 2014 and picking out those where it was simplest for
+          * in October 2014 and picking out those where it was simplest for
           * them to describe the situation by beginning with a "*.[tld]" rule
           * or every rule was "[something].[tld]".
           */
-         const char *const tlds[] = {"bd","bn","ck","cy","er","et","fj","fk",
+         const char *const tlds[] = {"bd","bn","ck","cy","er","fj","fk",
                                      "gu","il","jm","ke","kh","kw","mm","mz",
-                                     "ni","np","nz","pg","tr","uk","ye","za",
-                                     "zm","zw"};
+                                     "ni","np","pg","ye","za","zm","zw"};
          uint_t i, tld_num = sizeof(tlds) / sizeof(tlds[0]);
 
          for (i = 0; i < tld_num; i++) {
