@@ -3,7 +3,7 @@
  *
  * This server is an example. Play with it and modify to your taste.
  *
- * Copyright 2010 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright 2010-2015 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  * Debugging macros
  */
 #define _MSG(...)
-#define MSG(...)  printf("[vsource dpi]: " __VA_ARGS__)
+#define MSG(...)  fprintf(stderr, "[vsource dpi]: " __VA_ARGS__)
 
 /*---------------------------------------------------------------------------*/
 
@@ -42,38 +42,41 @@ void send_dpip_tag(Dsh *sh, char *dpip_tag)
 
 /*
  * Send source as plain text
+ * (handles embedded null chars correctly).
  */
 void send_plain_text(Dsh *sh, int data_size)
 {
-   int bytes_read = 0;
-   char *src_str;
+   char *token;
+   int bytes_read = 0, token_size;
 
    /* Send HTTP header for plain text MIME type */
    a_Dpip_dsh_write_str(sh, 0, "Content-type: text/plain\n\n");
 
    while (bytes_read < data_size &&
-          (src_str = a_Dpip_dsh_read_token(sh, 1))) {
-      bytes_read += strlen(src_str);
-      a_Dpip_dsh_write_str(sh, 1, src_str);
-      dFree(src_str);
+          (token = a_Dpip_dsh_read_token2(sh, 1, &token_size))) {
+      bytes_read += token_size;
+      _MSG("data_size=%d bytes_read=%d\n", data_size, bytes_read);
+      a_Dpip_dsh_write(sh, 1, token, token_size);
+      dFree(token);
    }
 }
 
 /*
  * Send source as plain text with line numbers
+ * (handles embedded null chars correctly).
  */
 void send_numbered_text(Dsh *sh, int data_size)
 {
-   int bytes_read = 0, line = 1;
-   char *p, *q, *src_str, line_str[32];
+   int bytes_read = 0, line = 1, token_size = 0;
+   char *p, *q, *token, line_str[32];
 
    /* Send HTTP header for plain text MIME type */
    a_Dpip_dsh_write_str(sh, 0, "Content-type: text/plain\n\n");
 
    while (bytes_read < data_size &&
-          (src_str = a_Dpip_dsh_read_token(sh, 1))) {
-      bytes_read += strlen(src_str);
-      p = q = src_str;
+          (token = a_Dpip_dsh_read_token2(sh, 1, &token_size))) {
+      bytes_read += token_size;
+      p = q = token;
 
       while (*p) {
          snprintf(line_str, 32, "%2d: ", line);
@@ -84,28 +87,30 @@ void send_numbered_text(Dsh *sh, int data_size)
                ++p;
             ++line;
          } else {
-            a_Dpip_dsh_write_str(sh, 1, q);
+            /* send all the rest */
+            a_Dpip_dsh_write(sh, 1, q, token_size - (q - token));
             break;
          }
          q = ++p;
       }
-      dFree(src_str);
+      dFree(token);
    }
 }
 
 /*
  * Send source as html text with line numbers
+ * (handles embedded null chars correctly).
  */
 void send_html_text(Dsh *sh, const char *url, int data_size)
 {
-   int bytes_read = 0, old_line = 0, line = 1;
-   char *p, *q, *src_str, line_str[128];
+   int bytes_read = 0, old_line = 0, line = 1, token_size = 0;
+   char *p, *q, *token, line_str[128];
 
    if (dStrnAsciiCasecmp(url, "dpi:", 4) == 0 &&
        strncmp(url+4, "/vsource/:", 10) == 0)
       url += 14;
 
-   /* Send HTTP header for plain text MIME type */
+   /* Send HTTP header for html text MIME type */
    a_Dpip_dsh_write_str(sh, 0, "Content-type: text/html\n\n");
 
    a_Dpip_dsh_write_str(sh, 0, DOCTYPE);
@@ -113,23 +118,25 @@ void send_html_text(Dsh *sh, const char *url, int data_size)
                      "\n"
                      "<html><head>\n"
                      "<title>Source for %s</title>\n"
-                     "<style type=\"text/css\">PRE {white-space: pre-wrap}\n"
+                     "<style type=\"text/css\">\n"
+                     " body {white-space: pre-wrap; font-family: monospace}\n"
+                     " td.r1 {background-color:#B87333}\n"
+                     " td.r2 {background-color:#DD7F32}\n"
                      "</style>\n"
                      "</head>\n"
                      "<body id=\"dillo_vs\">\n<table cellpadding='0'>\n", url);
 
    while (bytes_read < data_size &&
-          (src_str = a_Dpip_dsh_read_token(sh, 1))) {
-      bytes_read += strlen(src_str);
-      p = q = src_str;
+          (token = a_Dpip_dsh_read_token2(sh, 1, &token_size))) {
+      bytes_read += token_size;
+      p = q = token;
 
       while (*p) {
          if (line > old_line) {
             snprintf(line_str, 128,
-                     "%s<tr><td bgcolor='%s'>%d%s<td><pre>",
-                     (line > 1) ? "</pre>" : "",
-                     (line & 1) ? "#B87333" : "#DD7F32", line,
-                     (line == 1 || (line % 10) == 0) ? "&nbsp;&nbsp;" : "");
+                     "<tr><td class='%s'>%d%s<td>",
+                     (line & 1) ? "r1" : "r2", line,
+                     (line == 1 || (line % 10) == 0) ? "&nbsp;" : "");
             a_Dpip_dsh_write_str(sh, 0, line_str);
             old_line = line;
          }
@@ -143,17 +150,16 @@ void send_html_text(Dsh *sh, const char *url, int data_size)
                a_Dpip_dsh_write(sh, 0, q, p - q);
                a_Dpip_dsh_write_str(sh, 0, (*p == '<') ? "&lt;" : "&amp;");
             }
-         } else {
-            a_Dpip_dsh_write_str(sh, 1, q);
+         } else { 
+            /* send all the rest */
+            a_Dpip_dsh_write(sh, 1, q, token_size - (q - token));
             break;
          }
          q = ++p;
       }
-      dFree(src_str);
+      dFree(token);
    }
 
-   if (data_size > 0)
-      a_Dpip_dsh_write_str(sh, 0, "</pre>");
    a_Dpip_dsh_write_str(sh, 1, "</table></body></html>");
 }
 
@@ -194,7 +200,7 @@ int main(void)
     * asking from us. a_Dpip_dsh_read_token() will block and return
     * a full dpip token or null on error (it's commented in dpip.c) */
    dpip_tag = a_Dpip_dsh_read_token(sh, 1);
-   MSG("tag = [%s]\n", dpip_tag);
+   _MSG("tag = [%s]\n", dpip_tag);
 
    /* Now that we have the dpip_tag, let's isolate the command and url */
    cmd = a_Dpip_get_attr(dpip_tag, "cmd");
