@@ -336,7 +336,7 @@ int register_all(struct dp **attlist)
    char *user_dpidir = NULL, *sys_dpidir = NULL, *dpidrc = NULL;
    struct dirent *user_dirent, *sys_dirent;
    int st;
-   int snum, usr_srv_num;
+   int snum;
    size_t dp_sz = sizeof(struct dp);
 
    if (*attlist != NULL) {
@@ -370,7 +370,7 @@ int register_all(struct dp **attlist)
    }
 
    /* Get list of services in user's .dillo/dpi directory */
-   snum = usr_srv_num = 0;
+   snum = 0;
    if (user_dpidir && (user_dir_stream = opendir(user_dpidir)) != NULL) {
       while ((user_dirent = readdir(user_dir_stream)) != NULL) {
          if (user_dirent->d_name[0] == '.')
@@ -380,7 +380,6 @@ int register_all(struct dp **attlist)
          if (st == 0)
             snum++;
       }
-      usr_srv_num = snum;
       closedir(user_dir_stream);
    }
    if (sys_dpidir && (sys_dir_stream = opendir(sys_dpidir)) != NULL) {
@@ -763,7 +762,7 @@ void stop_active_dpis(struct dp *dpi_attr_list, int numdpis)
       } else if (CKD_WRITE(sock_fd, bye_cmd) == -1) {
          ERRMSG("stop_active_dpis", "write", errno);
       }
-      a_Misc_close_fd(sock_fd);
+      dClose(sock_fd);
    }
 
    dFree(auth_cmd);
@@ -783,7 +782,7 @@ void ignore_dpi_sockets(struct dp *dpi_attr_list, int numdpis)
 
    for (i = 0; i < numdpis; i++) {
       FD_CLR(dpi_attr_list[i].sock_fd, &sock_set);
-      a_Misc_close_fd(dpi_attr_list[i].sock_fd);
+      dClose(dpi_attr_list[i].sock_fd);
    }
 }
 
