@@ -20,6 +20,7 @@
 
 
 #include "alignedtextblock.hh"
+#include "../lout/debug.hh"
 #include <stdio.h>
 
 namespace dw {
@@ -62,6 +63,7 @@ int AlignedTextblock::CLASS_ID = -1;
 AlignedTextblock::AlignedTextblock (bool limitTextWidth):
    Textblock (limitTextWidth)
 {
+   DBG_OBJ_CREATE ("dw::AlignedTextblock");
    registerName ("dw::AlignedTextblock", &CLASS_ID);
 }
 
@@ -79,6 +81,7 @@ void AlignedTextblock::setRefTextblock (AlignedTextblock *ref)
 AlignedTextblock::~AlignedTextblock()
 {
    list->unref (listPos);
+   DBG_OBJ_DELETE ();
 }
 
 void AlignedTextblock::updateValue ()
