@@ -22,6 +22,7 @@
 #include "../lout/msg.h"
 #include "../lout/misc.hh"
 #include "../lout/unicode.hh"
+#include "../lout/debug.hh"
 
 #include <stdio.h>
 #include <math.h>
@@ -41,6 +42,103 @@ namespace dw {
 
 int Textblock::CLASS_ID = -1;
 
+Textblock::WordImgRenderer::WordImgRenderer (Textblock *textblock,
+                                             int wordNo)
+{
+   //printf ("new WordImgRenderer %p\n", this);
+
+   this->textblock = textblock;
+   this->wordNo = wordNo;
+   dataSet = false;
+}
+
+Textblock::WordImgRenderer::~WordImgRenderer ()
+{
+   //printf ("delete WordImgRenderer %p\n", this);
+}
+
+void Textblock::WordImgRenderer::setData (int xWordWidget, int lineNo)
+{
+   dataSet = true;
+   this->xWordWidget = xWordWidget;
+   this->lineNo = lineNo;
+}
+
+bool Textblock::WordImgRenderer::readyToDraw ()
+{
+   //print ();
+   //printf ("\n");
+
+   return dataSet && textblock->wasAllocated ()
+      && wordNo < textblock->words->size()
+      && lineNo < textblock->lines->size();
+}
+
+void Textblock::WordImgRenderer::getBgArea (int *x, int *y, int *width,
+                                            int *height)
+{
+   // TODO Subtract margin and border (padding box)?
+   Line *line = textblock->lines->getRef (lineNo);
+   *x = textblock->allocation.x + this->xWordWidget;
+   *y = textblock->lineYOffsetCanvas (line);
+   *width = textblock->words->getRef(wordNo)->size.width;
+   *height = line->boxAscent + line->boxDescent;
+}
+
+void Textblock::WordImgRenderer::getRefArea (int *xRef, int *yRef,
+                                             int *widthRef, int *heightRef)
+{
+   // See comment in Widget::drawBox about the reference area.
+   textblock->getPaddingArea (xRef, yRef, widthRef, heightRef);
+}
+
+core::style::Style *Textblock::WordImgRenderer::getStyle ()
+{
+   return textblock->words->getRef(wordNo)->style;
+}
+
+void Textblock::WordImgRenderer::draw (int x, int y, int width, int height)
+{
+   textblock->queueDrawArea (x - textblock->allocation.x,
+                             y - textblock->allocation.y, width, height);
+
+}
+
+void Textblock::WordImgRenderer::print ()
+{
+   printf ("%p: word #%d, ", this, wordNo);
+   if (wordNo < textblock->words->size())
+      textblock->printWordShort (textblock->words->getRef(wordNo));
+   else
+      printf ("<word %d does not exist>", wordNo);
+   printf (", data set: %s", dataSet ? "yes" : "no");
+}
+
+void Textblock::SpaceImgRenderer::getBgArea (int *x, int *y, int *width,
+                                             int *height)
+{
+   WordImgRenderer::getBgArea (x, y, width, height);
+   *x += *width;
+   *width = textblock->words->getRef(wordNo)->effSpace;
+}
+
+core::style::Style *Textblock::SpaceImgRenderer::getStyle ()
+{
+   return textblock->words->getRef(wordNo)->spaceStyle;
+}
+
+void Textblock::SpaceImgRenderer::print ()
+{
+   printf ("%p: word FOR SPACE #%d, ", this, wordNo);
+   if (wordNo < textblock->words->size())
+      textblock->printWordShort (textblock->words->getRef(wordNo));
+   else
+      printf ("<word %d does not exist>", wordNo);
+   printf (", data set: %s", dataSet ? "yes" : "no");
+}
+
+// ----------------------------------------------------------------------
+
 Textblock::DivChar Textblock::divChars[NUM_DIV_CHARS] = {
    // soft hyphen (U+00AD)
    { "\xc2\xad", true, false, true, PENALTY_HYPHEN, -1 },
@@ -79,6 +177,8 @@ int Textblock::penalties[PENALTY_NUM][2] = {
    { 100, 800 }
 };
 
+int Textblock::stretchabilityFactor = 100;
+
 /**
  * The character which is used to draw a hyphen at the end of a line,
  * either caused by automatic hyphenation, or by soft hyphens.
@@ -115,8 +215,14 @@ void Textblock::setPenaltyEmDashRight2 (int penaltyRightEmDash2)
    penalties[PENALTY_EM_DASH_RIGHT][1] = penaltyRightEmDash2;
 }
 
+void Textblock::setStretchabilityFactor (int stretchabilityFactor)
+{
+   Textblock::stretchabilityFactor = stretchabilityFactor;
+}
+
 Textblock::Textblock (bool limitTextWidth)
 {
+   DBG_OBJ_CREATE ("dw::Textblock");
    registerName ("dw::Textblock", &CLASS_ID);
    setFlags (BLOCK_LEVEL);
    setFlags (USES_HINTS);
@@ -181,8 +287,13 @@ Textblock::~Textblock ()
 
    for (int i = 0; i < words->size(); i++) {
       Word *word = words->getRef (i);
+
       if (word->content.type == core::Content::WIDGET)
          delete word->content.widget;
+
+      removeWordImgRenderer (i);
+      removeSpaceImgRenderer (i);
+
       word->style->unref ();
       word->spaceStyle->unref ();
    }
@@ -202,7 +313,7 @@ Textblock::~Textblock ()
       parent class destructor. (???) */
    words = NULL;
 
-   //DBG_OBJ_SET_NUM(this, "num_lines", lines->size ());
+   DBG_OBJ_DELETE ();
 }
 
 /**
@@ -800,8 +911,8 @@ void Textblock::calcWidgetSize (core::Widget *widget, core::Requisition *size)
          size->width = core::style::absLengthVal (wstyle->width)
             + wstyle->boxDiffWidth ();
       else
-         size->width = (int) (core::style::perLengthVal (wstyle->width)
-                       * availWidth);
+         size->width =
+            core::style::multiplyWithPerLength (availWidth, wstyle->width);
 
       if (wstyle->height == core::style::LENGTH_AUTO) {
          size->ascent = requisition.ascent;
@@ -813,9 +924,10 @@ void Textblock::calcWidgetSize (core::Widget *widget, core::Requisition *size)
                         + wstyle->boxDiffHeight ();
          size->descent = 0;
       } else {
-         double len = core::style::perLengthVal (wstyle->height);
-         size->ascent = (int) (len * availAscent);
-         size->descent = (int) (len * availDescent);
+         size->ascent =
+            core::style::multiplyWithPerLength (wstyle->height, availAscent);
+         size->descent =
+            core::style::multiplyWithPerLength (wstyle->height, availDescent);
       }
    }
 
@@ -1346,14 +1458,77 @@ Textblock::Word *Textblock::addWord (int width, int ascent, int descent,
                                      short flags, core::style::Style *style)
 {
    words->increase ();
-   Word *word = words->getLastRef ();
-   fillWord (word, width, ascent, descent, flags, style);
-   return word;
+   int wordNo = words->size () - 1;
+   initWord (wordNo);
+   fillWord (wordNo, width, ascent, descent, flags, style);
+   return words->getRef (wordNo);;
 }
 
-void Textblock::fillWord (Word *word, int width, int ascent, int descent,
+/**
+ * Basic initialization, which is neccessary before fillWord.
+ */
+void Textblock::initWord (int wordNo)
+{
+   Word *word = words->getRef (wordNo);
+
+   word->style = word->spaceStyle = NULL;
+   word->wordImgRenderer = NULL;
+   word->spaceImgRenderer = NULL;
+}
+
+void Textblock::removeWordImgRenderer (int wordNo)
+{
+   Word *word = words->getRef (wordNo);
+
+   if (word->style && word->wordImgRenderer) {
+      word->style->backgroundImage->removeExternalImgRenderer
+         (word->wordImgRenderer);
+      delete word->wordImgRenderer;
+      word->wordImgRenderer = NULL;
+   }
+}
+
+void Textblock::setWordImgRenderer (int wordNo)
+{
+   Word *word = words->getRef (wordNo);
+
+   if (word->style->backgroundImage) {
+      word->wordImgRenderer = new WordImgRenderer (this, wordNo);
+      word->style->backgroundImage->putExternalImgRenderer
+         (word->wordImgRenderer);
+   } else
+      word->wordImgRenderer = NULL;
+}
+
+void Textblock::removeSpaceImgRenderer (int wordNo)
+{
+   Word *word = words->getRef (wordNo);
+
+   if (word->spaceStyle && word->spaceImgRenderer) {
+      word->spaceStyle->backgroundImage->removeExternalImgRenderer
+         (word->spaceImgRenderer);
+      delete word->spaceImgRenderer;
+      word->spaceImgRenderer = NULL;
+   }
+}
+
+void Textblock::setSpaceImgRenderer (int wordNo)
+{
+   Word *word = words->getRef (wordNo);
+
+   if (word->spaceStyle->backgroundImage) {
+      word->spaceImgRenderer = new SpaceImgRenderer (this, wordNo);
+      word->spaceStyle->backgroundImage->putExternalImgRenderer
+         (word->spaceImgRenderer);
+   } else
+      word->spaceImgRenderer = NULL;
+}
+
+void Textblock::fillWord (int wordNo, int width, int ascent, int descent,
                           short flags, core::style::Style *style)
 {
+   Word *word = words->getRef (wordNo);
+
    word->size.width = width;
    word->size.ascent = ascent;
    word->size.descent = descent;
@@ -1363,8 +1538,15 @@ void Textblock::fillWord (Word *word, int width, int ascent, int descent,
    word->content.space = false;
    word->flags = flags;
 
+   removeWordImgRenderer (wordNo);
+   removeSpaceImgRenderer (wordNo);
+
    word->style = style;
    word->spaceStyle = style;
+
+   setWordImgRenderer (wordNo);
+   setSpaceImgRenderer (wordNo);
+
    style->ref ();
    style->ref ();
 }
@@ -1466,9 +1648,9 @@ void Textblock::calcTextSize (const char *text, size_t len,
       if (core::style::isAbsLength (style->lineHeight))
          height = core::style::absLengthVal(style->lineHeight);
       else
-         height = lout::misc::roundInt (
-                     core::style::perLengthVal(style->lineHeight) *
-                     style->font->size);
+         height =
+            core::style::multiplyWithPerLengthRounded (style->font->size,
+                                                       style->lineHeight);
       leading = height - style->font->size;
 
       size->ascent += leading / 2;
@@ -1478,10 +1660,13 @@ void Textblock::calcTextSize (const char *text, size_t len,
    /* In case of a sub or super script we increase the word's height and
     * potentially the line's height.
     */
-   if (style->valign == core::style::VALIGN_SUB)
-      size->descent += (style->font->ascent / 3);
-   else if (style->valign == core::style::VALIGN_SUPER)
-      size->ascent += (style->font->ascent / 2);
+   if (style->valign == core::style::VALIGN_SUB) {
+      int requiredDescent = style->font->descent + style->font->ascent / 3;
+      size->descent = misc::max (size->descent, requiredDescent);
+   } else if (style->valign == core::style::VALIGN_SUPER) {
+      int requiredAscent = style->font->ascent + style->font->ascent / 2;
+      size->ascent = misc::max (size->ascent, requiredAscent);
+   }
 }
 
 /**
@@ -1679,7 +1864,7 @@ void Textblock::addText (const char *text, size_t len,
             Word *word = words->getLastRef();
 
             setBreakOption (word, style, penalties[partPenaltyIndex[i]][0],
-                            penalties[partPenaltyIndex[i]][1]);
+                            penalties[partPenaltyIndex[i]][1], false);
 
             if (charRemoved[i])
                // Currently, only unconditional hyphens (UTF-8:
@@ -1747,6 +1932,7 @@ void Textblock::addText0 (const char *text, size_t len, short flags,
 
    Word *word = addWord (size->width, size->ascent, size->descent,
                          flags, style);
+   DBG_OBJ_ASSOC_CHILD (style);
    word->content.type = core::Content::TEXT;
    word->content.text = layout->textZone->strndup(text, len);
 
@@ -1839,7 +2025,7 @@ void Textblock::addSpace (core::style::Style *style)
 {
    int wordIndex = words->size () - 1;
    if (wordIndex >= 0) {
-      fillSpace (words->getRef(wordIndex), style);
+      fillSpace (wordIndex, style);
       accumulateWordData (wordIndex);
       correctLastWordExtremes ();
    }
@@ -1848,18 +2034,20 @@ void Textblock::addSpace (core::style::Style *style)
 /**
  * Add a break option (see setBreakOption() for details). Used instead
  * of addStyle for ideographic characters.
+ *
+ * When "forceBreak" is true, a break is even possible within PRE etc.
  */
-void Textblock::addBreakOption (core::style::Style *style)
+void Textblock::addBreakOption (core::style::Style *style, bool forceBreak)
 {
    int wordIndex = words->size () - 1;
    if (wordIndex >= 0) {
-      setBreakOption (words->getRef(wordIndex), style, 0, 0);
+      setBreakOption (words->getRef(wordIndex), style, 0, 0, forceBreak);
       // Call of accumulateWordData() is not needed here.
       correctLastWordExtremes ();
    }
 }
 
-void Textblock::fillSpace (Word *word, core::style::Style *style)
+void Textblock::fillSpace (int wordNo, core::style::Style *style)
 {
    // Old comment:
    // 
@@ -1875,11 +2063,13 @@ void Textblock::fillSpace (Word *word, core::style::Style *style)
    //
    // TODO: Re-evaluate again.
 
+   Word *word = words->getRef (wordNo);
+
    // TODO: This line does not work: addBreakOption (word, style);
 
    // Do not override a previously set break penalty.
    if (!word->content.space) {
-      setBreakOption (word, style, 0, 0);
+      setBreakOption (word, style, 0, 0, false);
 
       word->content.space = true;
       word->effSpace = word->origSpace = style->font->spaceWidth +
@@ -1892,9 +2082,14 @@ void Textblock::fillSpace (Word *word, core::style::Style *style)
       //DBG_OBJ_ARRSET_NUM (this, "words.%d.content.space", wordIndex,
       //                    word->content.space);
 
+
+      removeSpaceImgRenderer (wordNo);
+
       word->spaceStyle->unref ();
       word->spaceStyle = style;
       style->ref ();
+      
+      setSpaceImgRenderer (wordNo);
    }
 }
 
@@ -1904,26 +2099,37 @@ void Textblock::fillSpace (Word *word, core::style::Style *style)
  * alternative, e. g. for ideographic characters.
  */
 void Textblock::setBreakOption (Word *word, core::style::Style *style,
-                                int breakPenalty1, int breakPenalty2)
+                                int breakPenalty1, int breakPenalty2,
+                                bool forceBreak)
 {
    // TODO: lineMustBeBroken should be independent of the penalty
    // index? Otherwise, examine the last line.
    if (!word->badnessAndPenalty.lineMustBeBroken(0)) {
-      switch (style->whiteSpace) {
-      case core::style::WHITE_SPACE_NORMAL:
-      case core::style::WHITE_SPACE_PRE_LINE:
-      case core::style::WHITE_SPACE_PRE_WRAP:
+      if (forceBreak || isBreakAllowed (word))
          word->badnessAndPenalty.setPenalties (breakPenalty1, breakPenalty2);
-         break;
-
-      case core::style::WHITE_SPACE_PRE:
-      case core::style::WHITE_SPACE_NOWRAP:
+      else
          word->badnessAndPenalty.setPenalty (PENALTY_PROHIBIT_BREAK);
-         break;
-      }
    }
 }
 
+bool Textblock::isBreakAllowed (Word *word)
+{
+   switch (word->style->whiteSpace) {
+   case core::style::WHITE_SPACE_NORMAL:
+   case core::style::WHITE_SPACE_PRE_LINE:
+   case core::style::WHITE_SPACE_PRE_WRAP:
+      return true;
+
+   case core::style::WHITE_SPACE_PRE:
+   case core::style::WHITE_SPACE_NOWRAP:
+      return false;
+   }
+   
+   // compiler happiness
+   lout::misc::assertNotReached ();
+   return false;
+}
+
 
 /**
  * Cause a paragraph break
@@ -1995,6 +2201,7 @@ void Textblock::addParbreak (int space, core::style::Style *style)
    }
 
    word = addWord (0, 0, 0, 0, style);
+   DBG_OBJ_ASSOC_CHILD (style);
    word->content.type = core::Content::BREAK;
    word->badnessAndPenalty.setPenalty (PENALTY_FORCE_BREAK);
    word->content.breakSpace = space;
@@ -2018,6 +2225,8 @@ void Textblock::addLinebreak (core::style::Style *style)
       // ... otherwise, it has no size (and does not enlarge the line).
       word = addWord (0, 0, 0, 0, style);
 
+   DBG_OBJ_ASSOC_CHILD (style);
+
    word->content.type = core::Content::BREAK;
    word->badnessAndPenalty.setPenalty (PENALTY_FORCE_BREAK);
    word->content.breakSpace = 0;
