@@ -229,29 +229,36 @@ const char *const AboutSplash=
 "<table border='0' cellpadding='5' cellspacing='1' width='100%'>\n"
 "<tr>\n"
 " <td bgcolor='#CCCCCC'>\n"
-"  <h4>Release overview</h4>\n"
-"  December 24, 2014\n"
+"  <h4>Notes</h4>\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'>\n"
 "  <tr>\n"
 "   <td>\n"
-"<p>\n"
-"The dillo-3.0.4.1 release brings you fixes:\n"
 "<ul>\n"
-"<li> for linking with the recently-released fltk-1.3.3\n"
-" (we don't use <tt>fl_oldfocus</tt> anymore).\n"
-"<li> to make sure that windows are resizable with fltk-1.3.3.\n"
-"<li> not to load background images, or follow redirections or meta refresh,\n"
-"     in <tt>--local</tt> mode (security).\n"
-"<li> to permit linking on OS X (remove our <tt>Fl_Printer</tt> stub).\n"
-"<li> for a crash when searching from the address bar and no search urls are\n"
-"     found in dillorc.\n"
+" <li> Read the <a href='http://www.dillo.org/dillo3-help.html'>help</a>,\n"
+"      it's short.\n"
+" <li> There's a\n"
+"   <a href='http://www.dillo.org/dillorc'>dillorc</a>\n"
+"   (readable config) file inside the tarball. It is well-commented\n"
+"   and has plenty of options to customize dillo, so <STRONG>copy\n"
+"   it</STRONG> to your <STRONG>~/.dillo/</STRONG> directory, and\n"
+"   modify it to your taste.\n"
+" <li> The right mouse button brings up a context-sensitive menu\n"
+"   (available on pages, links, images, forms, the Back and Forward buttons,\n"
+"    and the bug meter).\n"
+" <li> Cookies are disabled by default for privacy. To log into certain\n"
+"   sites, you may need to <a href='http://www.dillo.org/Cookies.txt'>enable\n"
+"   cookies selectively</a>.\n"
+" <li> To stop third-party ads and tracking, you can use a\n"
+"   ~/.dillo/<a href='http://www.dillo.org/domainrc'>domainrc</a>/ file.\n"
+" <li> Frames, Java and Javascript are not supported.\n"
+" <li> This release is mainly intended for <strong>developers</strong>\n"
+"   and <strong>advanced users</strong>.\n"
+" <li> Documentation for developers is in the <CODE>/doc</CODE>\n"
+"   dir inside the tarball; you can find directions on everything\n"
+"   else at the home page.\n"
 "</ul>\n"
-"<p>\n"
-"...that shouldn't have to wait until dillo-3.1 is ready with its floating\n"
-"elements and assorted good things.\n"
-"<p>\n"
 "  </table>\n"
 "</table>\n"
 "</table>\n"
@@ -262,38 +269,42 @@ const char *const AboutSplash=
 "<table border='0' cellpadding='5' cellspacing='1' width='100%'>\n"
 "<tr>\n"
 " <td bgcolor='#CCCCCC'>\n"
-"  <h4>Notes</h4>\n"
+"  <h4>Release overview</h4>\n"
+"  June 30, 2015\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'>\n"
 "  <tr>\n"
 "   <td>\n"
+"<p>\n"
+"Among the changes in dillo-3.0.5 are:\n"
 "<ul>\n"
-" <li> There's a\n"
-"   <a href='http://www.dillo.org/dillorc'>dillorc</a>\n"
-"   (readable  config)  file inside the tarball. It is well-commented\n"
-"   and  has  plenty  of  options to customize dillo, so <STRONG>copy\n"
-"   it</STRONG>  to  your  <STRONG>~/.dillo/</STRONG>  directory, and\n"
-"   modify it to your taste.\n"
-" <li> Documentation for developers is in the <CODE>/doc</CODE>\n"
-"   dir  inside  the  tarball;  you can find directions on everything\n"
-"   else at the home page.\n"
-" <li> The right mouse button brings up a context-sensitive menu\n"
-"   (available on pages, links, images, forms, the Back and Forward buttons,\n"
-"    and the bug meter).\n"
-" <li> Dillo behaves very nicely when browsing local files, images, and HTML.\n"
-"   It's also very good for Internet searching.\n"
-" <li> This release is mainly intended for <strong>developers</strong>\n"
-"   and <strong>advanced users</strong>.\n"
-" <li> Frames, Java and Javascript are not supported.\n"
+"<li>Image buffer/cache improvements.\n"
+"<li>Fix for segfault when there's no dpid and view source is requested.\n"
+"<li>Fix view-source dpi to handle null characters correctly.\n"
+"<li>Made view-source dpi use CSS formatting (it's shorter and cleaner).\n"
+"<li>Crosscompile/buildroot-friendly fltk-config test.\n"
+"<li>Fix X11 icon name.\n"
+"<li>In location bar, tend toward showing beginning of URL instead of end.\n"
+"<li>Handle irix's version of vsnprintf().\n"
+"<li>INPUT, TEXTAREA placeholder attribute.\n"
+"<li>Better notification when user's domainrc settings block page\n"
+"    redirection.\n"
+"<li>Fix bug with font_factor preference and CSS font-size:(larger|smaller).\n"
+"<li>Recognize Menu key in keysrc.\n"
+"<li>HTTPS: change cipher list to \"ALL:!aNULL:!eNULL:!LOW:!EXPORT40:!RC4\",\n"
+"    disable SSL3, disable TLS compression.\n"
+"<li>Avoid requesting background images if an ancestor has display:none.\n"
+"<li>Ignore built-in search url if any are specified in dillorc.\n"
 "</ul>\n"
-"<br>\n"
+"<p>\n"
+"...that shouldn't have to wait until dillo-3.1 is ready with its floating\n"
+"elements and assorted good things.\n"
+"<p>\n"
 "  </table>\n"
 "</table>\n"
 "</table>\n"
 "\n"
-"<table border='0' width='100%' cellpadding='0' cellspacing='0'><tr><td height='10'></table>\n"
-"\n"
 "\n"
 "<!-- the main layout table, a small vertical spacer -->\n"
 "\n"
