@@ -239,7 +239,7 @@ static Dstr *Http_make_content_type(const DilloUrl *url)
    Dstr *dstr;
 
    if (URL_FLAGS(url) & URL_MultipartEnc) {
-      MSG("submitting multipart/form-data!\n");
+      _MSG("submitting multipart/form-data!\n");
       dstr = dStr_new("multipart/form-data; boundary=\"");
       if (URL_DATA(url)->len > 2) {
          /* boundary lines have "--" prepended. Skip that. */
@@ -262,13 +262,19 @@ static Dstr *Http_make_content_type(const DilloUrl *url)
  * Make the http query string
  */
 Dstr *a_Http_make_query_str(const DilloUrl *url, const DilloUrl *requester,
-                            bool_t use_proxy)
+                            int web_flags, bool_t use_proxy)
 {
    char *ptr, *cookies, *referer, *auth;
    Dstr *query      = dStr_new(""),
         *request_uri = dStr_new(""),
         *proxy_auth = dStr_new("");
 
+   /* BUG: dillo doesn't actually understand application/xml yet */
+   const char *accept_hdr_value =
+      web_flags & WEB_Image ? "image/png,image/*;q=0.8,*/*;q=0.5" :
+      web_flags & WEB_Stylesheet ? "text/css,*/*;q=0.1" :
+      "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8";
+
    if (use_proxy) {
       dStr_sprintfa(request_uri, "%s%s",
                     URL_STR(url),
@@ -294,23 +300,23 @@ Dstr *a_Http_make_query_str(const DilloUrl *url, const DilloUrl *requester,
       dStr_sprintfa(
          query,
          "POST %s HTTP/1.1\r\n"
-         "Connection: close\r\n"
-         "Accept: text/*,image/*,*/*;q=0.2\r\n"
-         "Accept-Charset: utf-8,*;q=0.8\r\n"
-         "Accept-Encoding: gzip\r\n"
-         "%s" /* language */
-         "%s" /* auth */
          "Host: %s\r\n"
-         "%s"
-         "%s"
          "User-Agent: %s\r\n"
-         "Content-Length: %ld\r\n"
+         "Accept: %s\r\n"
+         "%s" /* language */
+         "Accept-Encoding: gzip, deflate\r\n"
+         "%s" /* auth */
+         "DNT: 1\r\n"
+         "%s" /* proxy auth */
+         "%s" /* referer */
+         "Connection: close\r\n"
          "Content-Type: %s\r\n"
+         "Content-Length: %ld\r\n"
          "%s" /* cookies */
          "\r\n",
-         request_uri->str, HTTP_Language_hdr, auth ? auth : "",
-         URL_AUTHORITY(url), proxy_auth->str, referer, prefs.http_user_agent,
-         (long)URL_DATA(url)->len, content_type->str,
+         request_uri->str, URL_AUTHORITY(url), prefs.http_user_agent,
+         accept_hdr_value, HTTP_Language_hdr, auth ? auth : "",
+         proxy_auth->str, referer, content_type->str, (long)URL_DATA(url)->len,
          cookies);
       dStr_append_l(query, URL_DATA(url)->str, URL_DATA(url)->len);
       dStr_free(content_type, TRUE);
@@ -318,24 +324,25 @@ Dstr *a_Http_make_query_str(const DilloUrl *url, const DilloUrl *requester,
       dStr_sprintfa(
          query,
          "GET %s HTTP/1.1\r\n"
-         "%s"
-         "Connection: close\r\n"
-         "Accept: text/*,image/*,*/*;q=0.2\r\n"
-         "Accept-Charset: utf-8,*;q=0.8\r\n"
-         "Accept-Encoding: gzip\r\n"
-         "%s" /* language */
-         "%s" /* auth */
          "Host: %s\r\n"
-         "%s"
-         "%s"
          "User-Agent: %s\r\n"
+         "Accept: %s\r\n"
+         "%s" /* language */
+         "Accept-Encoding: gzip, deflate\r\n"
+         "%s" /* auth */
+         "DNT: 1\r\n"
+         "%s" /* proxy auth */
+         "%s" /* referer */
+         "Connection: close\r\n"
+         "%s" /* cache control */
          "%s" /* cookies */
          "\r\n",
-         request_uri->str,
+         request_uri->str, URL_AUTHORITY(url), prefs.http_user_agent,
+         accept_hdr_value, HTTP_Language_hdr, auth ? auth : "",
+         proxy_auth->str, referer,
          (URL_FLAGS(url) & URL_E2EQuery) ?
-            "Cache-Control: no-cache\r\nPragma: no-cache\r\n" : "",
-         HTTP_Language_hdr, auth ? auth : "", URL_AUTHORITY(url),
-         proxy_auth->str, referer, prefs.http_user_agent, cookies);
+            "Pragma: no-cache\r\nCache-Control: no-cache\r\n" : "",
+         cookies);
    }
    dFree(referer);
    dFree(cookies);
@@ -356,7 +363,7 @@ static void Http_send_query(ChainLink *Info, SocketData_t *S)
    DataBuf *dbuf;
 
    /* Create the query */
-   query = a_Http_make_query_str(S->web->url, S->web->requester,
+   query = a_Http_make_query_str(S->web->url, S->web->requester, S->web->flags,
                                  S->flags & HTTP_SOCKET_USE_PROXY);
    dbuf = a_Chain_dbuf_new(query->str, query->len, 0);
 
