@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  */
 
@@ -13,7 +13,6 @@
 #define __MIME_H__
 
 #include <config.h>
-#include <stddef.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -31,25 +30,18 @@ void *a_Html_text (const char *Type,void *web, CA_Callback_t *Call,
                        void **Data);
 void *a_Plain_text(const char *Type,void *web, CA_Callback_t *Call,
                        void **Data);
-#ifdef ENABLE_JPEG
-void *a_Jpeg_image(const char *Type,void *web, CA_Callback_t *Call,
-                       void **Data);
-#endif
-#ifdef ENABLE_PNG
-void *a_Png_image (const char *Type,void *web, CA_Callback_t *Call,
-                       void **Data);
-#endif
-#ifdef ENABLE_GIF
-void *a_Gif_image (const char *Type,void *web, CA_Callback_t *Call,
-                       void **Data);
-#endif
+void *a_Dicache_png_image (const char *Type,void *web, CA_Callback_t *Call,
+                           void **Data);
+void *a_Dicache_gif_image(const char *Type, void *Ptr, CA_Callback_t *Call,
+                          void **Data);
+void *a_Dicache_jpeg_image(const char *Type, void *Ptr, CA_Callback_t *Call,
+                           void **Data);
 
 /*
  * Functions defined inside Mime module
  */
 void a_Mime_init(void);
-void *a_Mime_set_viewer(const char *content_type, void *Ptr,
-                        CA_Callback_t *Call, void **Data);
+Viewer_t a_Mime_get_viewer(const char *content_type);
 
 #ifdef __cplusplus
 }
