@@ -62,6 +62,7 @@ static const struct key {
    { "darkgoldenrod", 0xb8860b},
    { "darkgray", 0xa9a9a9},
    { "darkgreen", 0x006400},
+   { "darkgrey", 0xa9a9a9},
    { "darkkhaki", 0xbdb76b},
    { "darkmagenta", 0x8b008b},
    { "darkolivegreen", 0x556b2f},
@@ -72,11 +73,13 @@ static const struct key {
    { "darkseagreen", 0x8fbc8f},
    { "darkslateblue", 0x483d8b},
    { "darkslategray", 0x2f4f4f},
+   { "darkslategrey", 0x2f4f4f},
    { "darkturquoise", 0x00ced1},
    { "darkviolet", 0x9400d3},
    { "deeppink", 0xff1493},
    { "deepskyblue", 0x00bfff},
    { "dimgray", 0x696969},
+   { "dimgrey", 0x696969},
    { "dodgerblue", 0x1e90ff},
    { "firebrick", 0xb22222},
    { "floralwhite", 0xfffaf0},
@@ -93,6 +96,7 @@ static const struct key {
    { "green", 0x008000},
 #ifdef EXTENDED_COLOR
    { "greenyellow", 0xadff2f},
+   { "grey", 0x808080},
    { "honeydew", 0xf0fff0},
    { "hotpink", 0xff69b4},
    { "indianred", 0xcd5c5c},
@@ -107,6 +111,7 @@ static const struct key {
    { "lightcoral", 0xf08080},
    { "lightcyan", 0xe0ffff},
    { "lightgoldenrodyellow", 0xfafad2},
+   { "lightgray", 0xd3d3d3},
    { "lightgreen", 0x90ee90},
    { "lightgrey", 0xd3d3d3},
    { "lightpink", 0xffb6c1},
@@ -114,6 +119,7 @@ static const struct key {
    { "lightseagreen", 0x20b2aa},
    { "lightskyblue", 0x87cefa},
    { "lightslategray", 0x778899},
+   { "lightslategrey", 0x778899},
    { "lightsteelblue", 0xb0c4de},
    { "lightyellow", 0xffffe0},
 #endif
@@ -178,6 +184,7 @@ static const struct key {
    { "skyblue", 0x87ceeb},
    { "slateblue", 0x6a5acd},
    { "slategray", 0x708090},
+   { "slategrey", 0x708090},
    { "snow", 0xfffafa},
    { "springgreen", 0x00ff7f},
    { "steelblue", 0x4682b4},
