@@ -19,6 +19,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+#include "lout/debug.hh"
 #include "msg.h"
 #include "colors.h"
 #include "html_common.hh"
@@ -27,7 +28,6 @@
 
 using namespace dw::core::style;
 
-#define DEBUG_MSG(A, B, ...) _MSG(B, __VA_ARGS__)
 #define MSG_CSS(A, ...) MSG(A, __VA_ARGS__)
 #define DEBUG_TOKEN_LEVEL   0
 #define DEBUG_PARSE_LEVEL   0
@@ -47,6 +47,14 @@ typedef struct {
    const char *const *enum_symbols;
 } CssPropertyInfo;
 
+static const char *const Css_background_attachment_enum_vals[] = {
+   "scroll", "fixed", NULL
+};
+
+static const char *const Css_background_repeat_enum_vals[] = {
+   "repeat", "repeat-x", "repeat-y", "no-repeat", NULL
+};
+
 static const char *const Css_border_collapse_enum_vals[] = {
    "separate", "collapse", NULL
 };
@@ -139,11 +147,14 @@ static const char *const Css_word_spacing_enum_vals[] = {
 };
 
 const CssPropertyInfo Css_property_info[CSS_PROPERTY_LAST] = {
-   {"background-attachment", {CSS_TYPE_UNUSED}, NULL},
+   {"background-attachment", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
+    Css_background_attachment_enum_vals},
    {"background-color", {CSS_TYPE_COLOR, CSS_TYPE_UNUSED}, NULL},
-   {"background-image", {CSS_TYPE_UNUSED}, NULL},
-   {"background-position", {CSS_TYPE_UNUSED}, NULL},
-   {"background-repeat", {CSS_TYPE_UNUSED}, NULL},
+   {"background-image", {CSS_TYPE_URI, CSS_TYPE_UNUSED}, NULL},
+   {"background-position", {CSS_TYPE_BACKGROUND_POSITION, CSS_TYPE_UNUSED},
+    NULL},
+   {"background-repeat", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
+    Css_background_repeat_enum_vals},
    {"border-bottom-color", {CSS_TYPE_ENUM, CSS_TYPE_COLOR, CSS_TYPE_UNUSED},
     Css_border_color_enum_vals},
    {"border-bottom-style", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
@@ -194,7 +205,7 @@ const CssPropertyInfo Css_property_info[CSS_PROPERTY_LAST] = {
     Css_font_variant_enum_vals},
    {"font-weight", {CSS_TYPE_ENUM, CSS_TYPE_FONT_WEIGHT, CSS_TYPE_UNUSED},
     Css_font_weight_enum_vals},
-   {"height", {CSS_TYPE_LENGTH_PERCENTAGE, CSS_TYPE_UNUSED}, NULL},
+   {"height", {CSS_TYPE_LENGTH_PERCENTAGE, CSS_TYPE_AUTO, CSS_TYPE_UNUSED}, NULL},
    {"left", {CSS_TYPE_UNUSED}, NULL},
    {"letter-spacing", {CSS_TYPE_ENUM, CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_UNUSED},
     Css_letter_spacing_enum_vals},
@@ -206,10 +217,14 @@ const CssPropertyInfo Css_property_info[CSS_PROPERTY_LAST] = {
     Css_list_style_position_enum_vals},
    {"list-style-type", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED},
     Css_list_style_type_enum_vals},
-   {"margin-bottom", {CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_UNUSED}, NULL},
-   {"margin-left", {CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_UNUSED}, NULL},
-   {"margin-right", {CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_UNUSED}, NULL},
-   {"margin-top", {CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_UNUSED}, NULL},
+   {"margin-bottom",
+    {CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_AUTO, CSS_TYPE_UNUSED}, NULL},
+   {"margin-left",
+    {CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_AUTO, CSS_TYPE_UNUSED}, NULL},
+   {"margin-right",
+    {CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_AUTO, CSS_TYPE_UNUSED}, NULL},
+   {"margin-top",
+    {CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_AUTO, CSS_TYPE_UNUSED}, NULL},
    {"marker-offset", {CSS_TYPE_UNUSED}, NULL},
    {"marks", {CSS_TYPE_UNUSED}, NULL},
    {"max-height", {CSS_TYPE_UNUSED}, NULL},
@@ -239,7 +254,7 @@ const CssPropertyInfo Css_property_info[CSS_PROPERTY_LAST] = {
    {"vertical-align",{CSS_TYPE_ENUM, CSS_TYPE_UNUSED},Css_vertical_align_vals},
    {"visibility", {CSS_TYPE_UNUSED}, NULL},
    {"white-space", {CSS_TYPE_ENUM, CSS_TYPE_UNUSED}, Css_white_space_vals},
-   {"width", {CSS_TYPE_LENGTH_PERCENTAGE, CSS_TYPE_UNUSED}, NULL},
+   {"width", {CSS_TYPE_LENGTH_PERCENTAGE, CSS_TYPE_AUTO, CSS_TYPE_UNUSED}, NULL},
    {"word-spacing", {CSS_TYPE_ENUM, CSS_TYPE_SIGNED_LENGTH, CSS_TYPE_UNUSED},
     Css_word_spacing_enum_vals},
    {"z-index", {CSS_TYPE_UNUSED}, NULL},
@@ -260,8 +275,9 @@ typedef struct {
       CSS_SHORTHAND_BORDER,     /* special, used for 'border' */
       CSS_SHORTHAND_FONT,       /* special, used for 'font' */
    } type;
-   const CssPropertyName * properties;/* CSS_SHORTHAND_MULTIPLE:
-                                       *   must be terminated by -1
+   const CssPropertyName *properties; /* CSS_SHORTHAND_MULTIPLE:
+                                       *   must be terminated by
+                                       *   CSS_PROPERTY_END 
                                        * CSS_SHORTHAND_DIRECTIONS:
                                        *   must have length 4
                                        * CSS_SHORTHAND_BORDERS:
@@ -276,14 +292,14 @@ const CssPropertyName Css_background_properties[] = {
    CSS_PROPERTY_BACKGROUND_REPEAT,
    CSS_PROPERTY_BACKGROUND_ATTACHMENT,
    CSS_PROPERTY_BACKGROUND_POSITION,
-   (CssPropertyName) - 1
+   CSS_PROPERTY_END
 };
 
 const CssPropertyName Css_border_bottom_properties[] = {
    CSS_PROPERTY_BORDER_BOTTOM_WIDTH,
    CSS_PROPERTY_BORDER_BOTTOM_STYLE,
    CSS_PROPERTY_BORDER_BOTTOM_COLOR,
-   (CssPropertyName) - 1
+   CSS_PROPERTY_END
 };
 
 const CssPropertyName Css_border_color_properties[4] = {
@@ -297,14 +313,14 @@ const CssPropertyName Css_border_left_properties[] = {
    CSS_PROPERTY_BORDER_LEFT_WIDTH,
    CSS_PROPERTY_BORDER_LEFT_STYLE,
    CSS_PROPERTY_BORDER_LEFT_COLOR,
-   (CssPropertyName) - 1
+   CSS_PROPERTY_END
 };
 
 const CssPropertyName Css_border_right_properties[] = {
    CSS_PROPERTY_BORDER_RIGHT_WIDTH,
    CSS_PROPERTY_BORDER_RIGHT_STYLE,
    CSS_PROPERTY_BORDER_RIGHT_COLOR,
-   (CssPropertyName) - 1
+   CSS_PROPERTY_END
 };
 
 const CssPropertyName Css_border_style_properties[] = {
@@ -318,7 +334,7 @@ const CssPropertyName Css_border_top_properties[] = {
    CSS_PROPERTY_BORDER_TOP_WIDTH,
    CSS_PROPERTY_BORDER_TOP_STYLE,
    CSS_PROPERTY_BORDER_TOP_COLOR,
-   (CssPropertyName) - 1
+   CSS_PROPERTY_END
 };
 
 const CssPropertyName Css_border_width_properties[] = {
@@ -332,7 +348,7 @@ const CssPropertyName Css_list_style_properties[] = {
    CSS_PROPERTY_LIST_STYLE_TYPE,
    CSS_PROPERTY_LIST_STYLE_POSITION,
    CSS_PROPERTY_LIST_STYLE_IMAGE,
-   (CssPropertyName) - 1
+   CSS_PROPERTY_END
 };
 
 const CssPropertyName Css_margin_properties[] = {
@@ -346,7 +362,7 @@ const CssPropertyName Css_outline_properties[] = {
    CSS_PROPERTY_OUTLINE_COLOR,
    CSS_PROPERTY_OUTLINE_STYLE,
    CSS_PROPERTY_OUTLINE_WIDTH,
-   (CssPropertyName) - 1
+   CSS_PROPERTY_END
 };
 
 const CssPropertyName Css_padding_properties[] = {
@@ -377,7 +393,7 @@ const CssPropertyName Css_font_properties[] = {
    CSS_PROPERTY_FONT_VARIANT,
    CSS_PROPERTY_FONT_WEIGHT,
    CSS_PROPERTY_FONT_FAMILY,
-   (CssPropertyName) - 1
+   CSS_PROPERTY_END
 };
 
 static const CssShorthandInfo Css_shorthand_info[] = {
@@ -419,6 +435,7 @@ static const CssShorthandInfo Css_shorthand_info[] = {
  * ---------------------------------------------------------------------- */
 
 CssParser::CssParser(CssContext *context, CssOrigin origin,
+                     const DilloUrl *baseUrl,
                      const char *buf, int buflen)
 {
    this->context = context;
@@ -428,6 +445,7 @@ CssParser::CssParser(CssContext *context, CssOrigin origin,
    this->bufptr = 0;
    this->spaceSeparated = false;
    this->withinBlock = false;
+   this->baseUrl = baseUrl;
 
    nextToken ();
 }
@@ -694,6 +712,15 @@ bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType *type)
          }
          break;
 
+      case CSS_TYPE_BACKGROUND_POSITION:
+         if (ttype == CSS_TK_SYMBOL &&
+             (dStrAsciiCasecmp(tval, "center") == 0 ||
+              dStrAsciiCasecmp(tval, "left") == 0 ||
+              dStrAsciiCasecmp(tval, "right") == 0 ||
+              dStrAsciiCasecmp(tval, "top") == 0 ||
+              dStrAsciiCasecmp(tval, "bottom") == 0))
+            return true;
+         // Fall Through (lenght and percentage)
       case CSS_TYPE_LENGTH_PERCENTAGE:
       case CSS_TYPE_LENGTH_PERCENTAGE_NUMBER:
       case CSS_TYPE_LENGTH:
@@ -701,9 +728,12 @@ bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType *type)
             return false;
          // Fall Through
       case CSS_TYPE_SIGNED_LENGTH:
-         if (ttype == CSS_TK_DECINT ||
-             ttype == CSS_TK_FLOAT ||
-             (ttype == CSS_TK_SYMBOL && dStrAsciiCasecmp(tval, "auto") == 0))
+         if (ttype == CSS_TK_DECINT || ttype == CSS_TK_FLOAT)
+            return true;
+         break;
+
+      case CSS_TYPE_AUTO:
+         if (ttype == CSS_TK_SYMBOL && dStrAsciiCasecmp(tval, "auto") == 0)
             return true;
          break;
 
@@ -734,6 +764,12 @@ bool CssParser::tokenMatchesProperty(CssPropertyName prop, CssValueType *type)
          }
          break;
 
+      case CSS_TYPE_URI:
+         if (ttype == CSS_TK_SYMBOL &&
+             dStrAsciiCasecmp(tval, "url") == 0)
+            return true;
+         break;
+
       case CSS_TYPE_UNUSED:
       case CSS_TYPE_INTEGER:
          /* Not used for parser values. */
@@ -826,7 +862,7 @@ bool CssParser::parseRgbColor(int32_t *c) {
 
 bool CssParser::parseValue(CssPropertyName prop,
                            CssValueType type,
-                           CssPropertyValue * val)
+                           CssPropertyValue *val)
 {
    CssLengthType lentype;
    bool found, ret = false;
@@ -927,13 +963,16 @@ bool CssParser::parseValue(CssPropertyName prop,
             ret = true;
 
          val->intVal = CSS_CREATE_LENGTH(fval, lentype);
-      } else if (ttype == CSS_TK_SYMBOL && !dStrAsciiCasecmp(tval, "auto")) {
-         ret = true;
-         val->intVal = CSS_LENGTH_TYPE_AUTO;
-         nextToken();
       }
       break;
 
+   case CSS_TYPE_AUTO:
+      assert (ttype == CSS_TK_SYMBOL && !dStrAsciiCasecmp(tval, "auto"));
+      ret = true;
+      val->intVal = CSS_LENGTH_TYPE_AUTO;
+      nextToken();
+      break;
+
    case CSS_TYPE_COLOR:
       if (ttype == CSS_TK_COLOR) {
          val->intVal = a_Color_parse(tval, -1, &err);
@@ -1004,6 +1043,107 @@ bool CssParser::parseValue(CssPropertyName prop,
       }
       break;
 
+   case CSS_TYPE_URI:
+      if (ttype == CSS_TK_SYMBOL &&
+          dStrAsciiCasecmp(tval, "url") == 0) {
+         val->strVal = parseUrl();
+         nextToken();
+         if (val->strVal)
+            ret = true;
+      }
+      break;
+
+   case CSS_TYPE_BACKGROUND_POSITION:
+      // 'background-position' consists of one or two values: vertical and
+      // horizontal position; in most cases in this order. However, as long it
+      // is unambigous, the order can be switched: "10px left" and "left 10px"
+      // are both possible and have the same effect. For this reason, all
+      // possibilities are tested in parallel.
+
+      bool h[2], v[2];
+      int pos[2];
+      h[0] = v[0] = h[1] = v[1] = false;
+
+      // First: collect values in pos[0] and pos[1], and determine whether
+      // they can be used for a horizontal (h[i]) or vertical (v[i]) position
+      // (or both). When neither h[i] or v[i] is set, pos[i] is undefined.
+      for (i = 0; i < 2; i++) {
+         CssValueType typeTmp;
+         // tokenMatchesProperty will, for CSS_PROPERTY_BACKGROUND_POSITION,
+         // work on both parts, since they are exchangable.
+         if (tokenMatchesProperty (CSS_PROPERTY_BACKGROUND_POSITION,
+                                   &typeTmp)) {
+            h[i] = ttype != CSS_TK_SYMBOL ||
+               (dStrAsciiCasecmp(tval, "top") != 0 &&
+                dStrAsciiCasecmp(tval, "bottom") != 0);
+            v[i] = ttype != CSS_TK_SYMBOL ||
+               (dStrAsciiCasecmp(tval, "left") != 0 &&
+                dStrAsciiCasecmp(tval, "right") != 0);
+         } else
+            // No match.
+            h[i] = v[i] = false;
+
+         if (h[i] || v[i]) {
+            // Calculate values.
+            if (ttype == CSS_TK_SYMBOL) {
+               if (dStrAsciiCasecmp(tval, "top") == 0 ||
+                   dStrAsciiCasecmp(tval, "left") == 0) {
+                  pos[i] = CSS_CREATE_LENGTH (0.0, CSS_LENGTH_TYPE_PERCENTAGE);
+                  nextToken();
+               } else if (dStrAsciiCasecmp(tval, "center") == 0) {
+                  pos[i] = CSS_CREATE_LENGTH (0.5, CSS_LENGTH_TYPE_PERCENTAGE);
+                  nextToken();
+               } else if (dStrAsciiCasecmp(tval, "bottom") == 0 ||
+                          dStrAsciiCasecmp(tval, "right") == 0) {
+                  pos[i] = CSS_CREATE_LENGTH (1.0, CSS_LENGTH_TYPE_PERCENTAGE);
+                  nextToken();
+               } else
+                  // tokenMatchesProperty should have returned "false" already.
+                  lout::misc::assertNotReached ();
+            } else {
+               // We can assume <length> or <percentage> here ...
+               CssPropertyValue valTmp;
+               if (parseValue(prop, CSS_TYPE_LENGTH_PERCENTAGE, &valTmp)) {
+                  pos[i] = valTmp.intVal;
+                  ret = true;
+               } else
+                  // ... but something may still fail.
+                  h[i] = v[i] = false;
+            }
+         }
+
+         // If the first value cannot be read, do not read the second.
+         if (!h[i] && !v[i])
+            break;
+      }
+
+      // Second: Create the final value. Order will be determined here.
+      if (v[0] || h[0]) {
+         // If second value is not set, it is set to "center", i. e. 50%, (see
+         // CSS specification), which is suitable for both dimensions.
+         if (!h[1] && !v[1]) {
+            pos[1] = CSS_CREATE_LENGTH (0.5, CSS_LENGTH_TYPE_PERCENTAGE);
+            h[1] = v[1] = true;
+         }
+
+         // Only valid, when a combination h/v or v/h is possible.
+         if ((h[0] && v[1]) || (v[0] && h[1])) {
+            ret = true;
+            val->posVal = dNew(CssBackgroundPosition, 1);
+
+            // Prefer combination h/v:
+            if (h[0] && v[1]) {
+                val->posVal->posX = pos[0];
+                val->posVal->posY = pos[1];
+            } else {
+               // This should be v/h:
+                val->posVal->posX = pos[1];
+                val->posVal->posY = pos[0];
+            }
+         }
+      }
+      break;
+
    case CSS_TYPE_UNUSED:
       /* nothing */
       break;
@@ -1050,8 +1190,8 @@ static int Css_shorthand_info_cmp(const void *a, const void *b)
                       ((CssShorthandInfo *) b)->symbol);
 }
 
-void CssParser::parseDeclaration(CssPropertyList * props,
-                                 CssPropertyList * importantProps)
+void CssParser::parseDeclaration(CssPropertyList *props,
+                                 CssPropertyList *importantProps)
 {
    CssPropertyInfo pi = {NULL, {CSS_TYPE_UNUSED}, NULL}, *pip;
    CssShorthandInfo *sip;
@@ -1111,7 +1251,8 @@ void CssParser::parseDeclaration(CssPropertyList * props,
                   do {
                      for (found = false, i = 0;
                           !found &&
-                          Css_shorthand_info[sh_index].properties[i] != -1;
+                          Css_shorthand_info[sh_index].properties[i] !=
+                          CSS_PROPERTY_END;
                           i++)
                         if (tokenMatchesProperty(Css_shorthand_info[sh_index].
                                                  properties[i], &type)) {
@@ -1419,15 +1560,17 @@ char * CssParser::parseUrl()
    }
 
    if (urlStr) {
-      char *url = urlStr->str;
-      dStr_free(urlStr, 0);
+      DilloUrl *dilloUrl = a_Url_new(urlStr->str, a_Url_str(this->baseUrl));
+      char *url = dStrdup(a_Url_str(dilloUrl));
+      a_Url_free(dilloUrl);
+      dStr_free(urlStr, 1);
       return url;
    } else {
       return NULL;
    }
 }
 
-void CssParser::parseImport(DilloHtml *html, DilloUrl *baseUrl)
+void CssParser::parseImport(DilloHtml *html)
 {
    char *urlStr = NULL;
    bool importSyntaxIsOK = false;
@@ -1473,8 +1616,8 @@ void CssParser::parseImport(DilloHtml *html, DilloUrl *baseUrl)
    if (urlStr) {
       if (importSyntaxIsOK && mediaIsSelected) {
          MSG("CssParser::parseImport(): @import %s\n", urlStr);
-         DilloUrl *url = a_Html_url_new (html, urlStr, a_Url_str(baseUrl),
-                                         baseUrl ? 1 : 0);
+         DilloUrl *url = a_Html_url_new (html, urlStr, a_Url_str(this->baseUrl),
+                                         this->baseUrl ? 1 : 0);
          a_Html_load_stylesheet(html, url);
          a_Url_free(url);
       }
@@ -1566,11 +1709,12 @@ void CssParser::ignoreStatement()
    }
 }
 
-void CssParser::parse(DilloHtml *html, DilloUrl *url, CssContext * context,
+void CssParser::parse(DilloHtml *html, const DilloUrl *baseUrl,
+                      CssContext *context,
                       const char *buf,
                       int buflen, CssOrigin origin)
 {
-   CssParser parser (context, origin, buf, buflen);
+   CssParser parser (context, origin, baseUrl, buf, buflen);
    bool importsAreAllowed = true;
 
    while (parser.ttype != CSS_TK_END) {
@@ -1581,7 +1725,7 @@ void CssParser::parse(DilloHtml *html, DilloUrl *url, CssContext * context,
             if (dStrAsciiCasecmp(parser.tval, "import") == 0 &&
                 html != NULL &&
                 importsAreAllowed) {
-               parser.parseImport(html, url);
+               parser.parseImport(html);
             } else if (dStrAsciiCasecmp(parser.tval, "media") == 0) {
                parser.parseMedia();
             } else {
@@ -1597,11 +1741,12 @@ void CssParser::parse(DilloHtml *html, DilloUrl *url, CssContext * context,
    }
 }
 
-void CssParser::parseDeclarationBlock(const char *buf, int buflen,
+void CssParser::parseDeclarationBlock(const DilloUrl *baseUrl,
+                                      const char *buf, int buflen,
                                       CssPropertyList *props,
                                       CssPropertyList *propsImortant)
 {
-   CssParser parser (NULL, CSS_ORIGIN_AUTHOR, buf, buflen);
+   CssParser parser (NULL, CSS_ORIGIN_AUTHOR, baseUrl, buf, buflen);
 
    parser.withinBlock = true;
 
