@@ -343,7 +343,7 @@ void Html_tag_open_form(DilloHtml *html, const char *tag, int tagsize)
    HT2TB(html)->addParbreak (9, html->wordStyle ());
 
    if (html->InFlags & IN_FORM) {
-      BUG_MSG("nested forms\n");
+      BUG_MSG("Nested <form>.");
       return;
    }
    html->InFlags |= IN_FORM;
@@ -356,14 +356,14 @@ void Html_tag_open_form(DilloHtml *html, const char *tag, int tagsize)
       if (!dStrAsciiCasecmp(attrbuf, "post")) {
          method = DILLO_HTML_METHOD_POST;
       } else if (dStrAsciiCasecmp(attrbuf, "get")) {
-         BUG_MSG("Unknown form submission method \"%s\"\n", attrbuf);
+         BUG_MSG("<form> submission method unknown: '%s'.", attrbuf);
       }
    }
    if ((attrbuf = a_Html_get_attr(html, tag, tagsize, "action")))
       action = a_Html_url_new(html, attrbuf, NULL, 0);
    else {
       if (html->DocType != DT_HTML || html->DocTypeVersion <= 4.01f)
-         BUG_MSG("action attribute is required for <form>\n");
+         BUG_MSG("<form> requires action attribute.");
       action = a_Url_dup(html->base_url);
    }
    content_type = DILLO_HTML_ENC_URLENCODED;
@@ -417,7 +417,7 @@ static int Html_input_get_size(DilloHtml *html, const char *attrbuf)
       if (size < 1 || size > MAX_SIZE) {
          int badSize = size;
          size = (size < 1 ? 20 : MAX_SIZE);
-         BUG_MSG("input size=%d, using size=%d instead\n", badSize, size);
+         BUG_MSG("<input> size=%d, using size=%d instead.", badSize, size);
       }
    }
    return size;
@@ -431,17 +431,17 @@ void Html_tag_open_input(DilloHtml *html, const char *tag, int tagsize)
    DilloHtmlInputType inp_type;
    Resource *resource = NULL;
    Embed *embed = NULL;
-   char *value, *name, *type, *init_str;
+   char *value, *name, *type, *init_str, *placeholder = NULL;
    const char *attrbuf, *label;
    bool init_val = false;
    ResourceFactory *factory;
 
    if (html->InFlags & IN_SELECT) {
-      BUG_MSG("<input> element inside <select>\n");
+      BUG_MSG("<input> inside <select>.");
       return;
    }
    if (html->InFlags & IN_BUTTON) {
-      BUG_MSG("<input> element inside <button>\n");
+      BUG_MSG("<input> inside <button>.");
       return;
    }
 
@@ -456,9 +456,10 @@ void Html_tag_open_input(DilloHtml *html, const char *tag, int tagsize)
    inp_type = DILLO_HTML_INPUT_UNKNOWN;
    if (!dStrAsciiCasecmp(type, "password")) {
       inp_type = DILLO_HTML_INPUT_PASSWORD;
+      placeholder = a_Html_get_attr_wdef(html, tag,tagsize,"placeholder",NULL);
       attrbuf = a_Html_get_attr(html, tag, tagsize, "size");
       int size = Html_input_get_size(html, attrbuf);
-      resource = factory->createEntryResource (size, true, NULL);
+      resource = factory->createEntryResource (size, true, NULL, placeholder);
       init_str = value;
    } else if (!dStrAsciiCasecmp(type, "checkbox")) {
       inp_type = DILLO_HTML_INPUT_CHECKBOX;
@@ -478,7 +479,7 @@ void Html_tag_open_input(DilloHtml *html, const char *tag, int tagsize)
       inp_type = DILLO_HTML_INPUT_HIDDEN;
       init_str = value;
       int size = Html_input_get_size(html, NULL);
-      resource = factory->createEntryResource(size, false, name);
+      resource = factory->createEntryResource(size, false, name, NULL);
    } else if (!dStrAsciiCasecmp(type, "submit")) {
       inp_type = DILLO_HTML_INPUT_SUBMIT;
       init_str = (value) ? value : dStrdup("submit");
@@ -507,12 +508,12 @@ void Html_tag_open_input(DilloHtml *html, const char *tag, int tagsize)
          DilloHtmlForm *form = html->getCurrentForm();
          if (form->method != DILLO_HTML_METHOD_POST) {
             valid = false;
-            BUG_MSG("Forms with file input MUST use HTTP POST method\n");
+            BUG_MSG("<form> with file input MUST use HTTP POST method.");
             MSG("File input ignored in form not using HTTP POST method\n");
          } else if (form->content_type != DILLO_HTML_ENC_MULTIPART) {
             valid = false;
-            BUG_MSG("Forms with file input MUST use multipart/form-data"
-                    " encoding\n");
+            BUG_MSG("<form> with file input MUST use multipart/form-data"
+                    " encoding.");
             MSG("File input ignored in form not using multipart/form-data"
                 " encoding\n");
          }
@@ -531,9 +532,10 @@ void Html_tag_open_input(DilloHtml *html, const char *tag, int tagsize)
    } else {
       /* Text input, which also is the default */
       inp_type = DILLO_HTML_INPUT_TEXT;
+      placeholder = a_Html_get_attr_wdef(html, tag,tagsize,"placeholder",NULL);
       attrbuf = a_Html_get_attr(html, tag, tagsize, "size");
       int size = Html_input_get_size(html, attrbuf);
-      resource = factory->createEntryResource(size, false, NULL);
+      resource = factory->createEntryResource(size, false, NULL, placeholder);
       init_str = value;
    }
    if (resource)
@@ -573,6 +575,7 @@ void Html_tag_open_input(DilloHtml *html, const char *tag, int tagsize)
    dFree(name);
    if (init_str != value)
       dFree(init_str);
+   dFree(placeholder);
    dFree(value);
 }
 
@@ -606,7 +609,8 @@ void Html_tag_open_isindex(DilloHtml *html, const char *tag, int tagsize)
       HT2TB(html)->addText(attrbuf, html->wordStyle ());
 
    ResourceFactory *factory = HT2LT(html)->getResourceFactory();
-   EntryResource *entryResource = factory->createEntryResource (20,false,NULL);
+   EntryResource *entryResource = factory->createEntryResource (20, false,
+                                                                NULL, NULL);
    embed = new Embed (entryResource);
    Html_add_input(html, DILLO_HTML_INPUT_INDEX, embed, NULL, NULL, FALSE);
 
@@ -641,33 +645,35 @@ void Html_tag_content_textarea(DilloHtml *html, const char *tag, int tagsize)
       cols = strtol(attrbuf, NULL, 10);
    } else {
       if (html->DocType != DT_HTML || html->DocTypeVersion <= 4.01f)
-         BUG_MSG("cols attribute is required for <textarea>\n");
+         BUG_MSG("<textarea> requires cols attribute.");
       cols = 20;
    }
    if (cols < 1 || cols > MAX_COLS) {
       int badCols = cols;
       cols = (cols < 1 ? 20 : MAX_COLS);
-      BUG_MSG("textarea cols=%d, using cols=%d instead\n", badCols, cols);
+      BUG_MSG("<textarea> cols=%d, using cols=%d instead.", badCols, cols);
    }
    if ((attrbuf = a_Html_get_attr(html, tag, tagsize, "rows"))) {
       rows = strtol(attrbuf, NULL, 10);
    } else {
       if (html->DocType != DT_HTML || html->DocTypeVersion <= 4.01f)
-         BUG_MSG("rows attribute is required for <textarea>\n");
+         BUG_MSG("<textarea> requires rows attribute.");
       rows = 10;
    }
    if (rows < 1 || rows > MAX_ROWS) {
       int badRows = rows;
       rows = (rows < 1 ? 2 : MAX_ROWS);
-      BUG_MSG("textarea rows=%d, using rows=%d instead\n", badRows, rows);
+      BUG_MSG("<textarea> rows=%d, using rows=%d instead.", badRows, rows);
    }
    name = NULL;
    if ((attrbuf = a_Html_get_attr(html, tag, tagsize, "name")))
       name = dStrdup(attrbuf);
 
+   attrbuf = a_Html_get_attr(html, tag, tagsize, "placeholder");
+
    ResourceFactory *factory = HT2LT(html)->getResourceFactory();
    MultiLineTextResource *textres =
-      factory->createMultiLineTextResource (cols, rows);
+      factory->createMultiLineTextResource (cols, rows, attrbuf);
 
    Embed *embed = new Embed(textres);
    /* Readonly or not? */
@@ -799,11 +805,11 @@ void Html_tag_close_select(DilloHtml *html)
 void Html_tag_open_optgroup(DilloHtml *html, const char *tag, int tagsize)
 {
    if (!(html->InFlags & IN_SELECT)) {
-      BUG_MSG("<optgroup> element outside <select>\n");
+      BUG_MSG("<optgroup> outside <select>.");
       return;
    }
    if (html->InFlags & IN_OPTGROUP) {
-      BUG_MSG("nested <optgroup>\n");
+      BUG_MSG("Nested <optgroup>.");
       return;
    }
    if (html->InFlags & IN_OPTION) {
@@ -821,7 +827,7 @@ void Html_tag_open_optgroup(DilloHtml *html, const char *tag, int tagsize)
       bool enabled = (a_Html_get_attr(html, tag, tagsize, "disabled") == NULL);
 
       if (!label) {
-         BUG_MSG("label attribute is required for <optgroup>\n");
+         BUG_MSG("<optgroup> requires label attribute.");
          label = strdup("");
       }
 
@@ -859,7 +865,7 @@ void Html_tag_close_optgroup(DilloHtml *html)
 void Html_tag_open_option(DilloHtml *html, const char *tag, int tagsize)
 {
    if (!(html->InFlags & IN_SELECT)) {
-      BUG_MSG("<option> element outside <select>\n");
+      BUG_MSG("<option> outside <select>.");
       return;
    }
    if (html->InFlags & IN_OPTION)
@@ -918,7 +924,7 @@ void Html_tag_open_button(DilloHtml *html, const char *tag, int tagsize)
       inp_type = DILLO_HTML_INPUT_BUTTON_SUBMIT;
    } else {
       inp_type = DILLO_HTML_INPUT_UNKNOWN;
-      BUG_MSG("Unknown button type: \"%s\"\n", type);
+      BUG_MSG("<button> type unknown: '%s'.", type);
    }
 
    if (inp_type != DILLO_HTML_INPUT_UNKNOWN) {
